import argparse
import os
import sys
from typing import List
from .download import EmbeddingsDownloader
from .operations import EmbeddingsOperator
from .utils import get_supported_languages

def print_welcome_logo():
    """Print a welcome logo for the CLI"""
    logo = [
        "///-------------------------------------------///",
        "///                                           ///",
        "///   🧌 GrEmLIn                              ///",
        "///   Multilingual Embeddings Toolkit         ///",
        "///                                           ///",
        "///-------------------------------------------///"
    ]
    for line in logo:
        print(line)
    print("\n")

def print_extended_help():
    """Print comprehensive help information"""
    print_welcome_logo()
    help_text = [
        "Available Commands:",
        "  download   Download embeddings for specified languages",
        "  merge      Merge different types of embeddings",
        "  languages  List all supported languages",
        "",
        "Download Command Usage:",
        "  gremlin download -l yo sw -t glove -o ./embeddings",
        "  - -l/--languages: List of language codes (required)",
        "  - -t/--type: Embedding type (glove or graph, required)",
        "  - -o/--output: Output directory (default: ./embeddings)",
        "",
        "Merge Command Usage:",
        "  gremlin merge -g glove_file.txt -p graph_file.txt -o merged_file.txt",
        "  - -g/--glove: Path to GloVe embeddings file (required)",
        "  - -p/--graph: Path to graph embeddings file (required)",
        "  - -o/--output: Output path for merged embeddings (required)",
        "",
        "Languages Command Usage:",
        "  gremlin languages [--conceptnet-only] [--codes-only]",
        "  - --conceptnet-only: Show only languages available in ConceptNet",
        "  - --codes-only: Show only language codes",
        "",
        "Examples:",
        "  Download GloVe embeddings for Yoruba and Swahili:",
        "    gremlin download -l yo sw -t glove",
        "  Merge embeddings from two sources:",
        "    gremlin merge -g glove.txt -p graph.txt -o merged.txt",
        "  List all supported languages:",
        "    gremlin languages",
        "",
    ]
    print("\n".join(help_text))

def print_languages(conceptnet_only: bool = False, codes_only: bool = False):
    """Print the list of supported languages."""
    languages = get_supported_languages()
    
    if conceptnet_only:
        languages = [lang for lang in languages if lang[4] == "✓"]
    
    if codes_only:
        for lang in languages:
            print(lang[0])
        return
    
    # Print header
    print(f"{'Code':<8} {'Language':<20} {'Corpus Size':<12} {'Level':<8} {'ConceptNet':<10}")
    print("-" * 60)
    
    # Print languages
    for code, name, size, level, availability in languages:
        print(f"{code:<8} {name:<20} {size:<12} {level:<8} {availability:<10}")

def download_embeddings(lang_codes: List[str], embedding_type: str, output_dir: str):
    """Download embeddings for specified languages."""
    downloader = EmbeddingsDownloader(cache_dir=output_dir)
    
    for lang in lang_codes:
        try:
            if embedding_type == "glove":
                path = downloader.download_glove(lang, "txt")
                print(f"Downloaded GloVe embeddings for {lang} to {path}")
            elif embedding_type == "graph":
                path = downloader.download_graph(lang)
                print(f"Downloaded graph embeddings for {lang} to {path}")
            else:
                print(f"Unknown embedding type: {embedding_type}")
                return
        except Exception as e:
            print(f"Error downloading {embedding_type} embeddings for {lang}: {str(e)}")

def merge_embeddings(glove_path: str, graph_path: str, output_path: str):
    """Merge GloVe and graph embeddings."""
    operator = EmbeddingsOperator()
    
    try:
        # Load embeddings
        print("Loading GloVe embeddings...")
        glove_embeddings = operator.read_embeddings_from_text(glove_path)
        
        print("Loading Graph embeddings...")
        graph_embeddings = operator.read_embeddings_from_text(graph_path)
        
        # Align embeddings
        print("Aligning embeddings...")
        aligned_embeddings = operator.svd_alignment(glove_embeddings, graph_embeddings)
        
        # Save results
        print(f"Saving aligned embeddings to {output_path}...")
        operator.save_embeddings_to_text(aligned_embeddings, output_path)
        
        print("Merging complete!")
    except Exception as e:
        print(f"Error during merging: {str(e)}")

def main():
    parser = argparse.ArgumentParser(
        description="GrEmLIn: Multilingual Embeddings Toolkit",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog="\nRun with --help for more detailed information."
    )
    subparsers = parser.add_subparsers(dest="command", help="Available commands")
    
    # Download command
    download_parser = subparsers.add_parser("download", help="Download embeddings")
    download_parser.add_argument(
        "--languages", "-l",
        nargs="+",
        required=True,
        help="Language codes to download (e.g., en fr de)"
    )
    download_parser.add_argument(
        "--type", "-t",
        choices=["glove", "graph"],
        required=True,
        help="Type of embeddings to download"
    )
    download_parser.add_argument(
        "--output", "-o",
        default="./embeddings",
        help="Output directory for downloaded files"
    )
    
    # Merge command
    merge_parser = subparsers.add_parser("merge", help="Merge GloVe and graph embeddings")
    merge_parser.add_argument(
        "--glove", "-g",
        required=True,
        help="Path to GloVe embeddings file"
    )
    merge_parser.add_argument(
        "--graph", "-p",
        required=True,
        help="Path to graph embeddings file"
    )
    merge_parser.add_argument(
        "--output", "-o",
        required=True,
        help="Output path for merged embeddings"
    )
    
    # Add extended help option
    parser.add_argument(
        "--detailed-help", "-d",
        action="store_true",
        help="Show detailed help and usage examples"
    )

    # Languages command
    languages_parser = subparsers.add_parser("languages", help="List supported languages")
    languages_parser.add_argument(
        "--conceptnet-only",
        action="store_true",
        help="Show only languages available in ConceptNet"
    )
    languages_parser.add_argument(
        "--codes-only",
        action="store_true",
        help="Show only language codes"
    )
    
    args = parser.parse_args()
    
    # Check for detailed help
    if args.detailed_help:
        print_extended_help()
        sys.exit(0)
    
    if args.command == "download":
        os.makedirs(args.output, exist_ok=True)
        download_embeddings(args.languages, args.type, args.output)
    elif args.command == "merge":
        merge_embeddings(args.glove, args.graph, args.output)
    elif args.command == "languages":
        print_languages(args.conceptnet_only, args.codes_only)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()