import os
import requests
from huggingface_hub import hf_hub_download
from typing import List, Optional
from .utils import get_available_languages, get_language_name

class EmbeddingsDownloader:
    """Handle downloading of embeddings from Hugging Face repos."""
    
    GLOVE_REPO_TEMPLATE = "DFKI/glove-{}-cc100"
    GRAPH_REPO = "DGurgurov/conceptnet_embeddings"
    
    def __init__(self, cache_dir: Optional[str] = None):
        self.cache_dir = cache_dir or os.path.expanduser("~/.cache/gremlin")
        os.makedirs(self.cache_dir, exist_ok=True)
        self.available_languages = [lang[0] for lang in get_available_languages()]
    
    def _validate_language(self, lang_code: str):
        """Validate if the language is supported."""
        if lang_code not in self.available_languages:
            # Prepare a helpful error message
            similar_langs = self._find_similar_languages(lang_code)
            
            if similar_langs:
                similar_str = ", ".join([f"{code} ({get_language_name(code)})" for code in similar_langs])
                raise ValueError(f"Language code '{lang_code}' is not supported. Did you mean: {similar_str}?")
            else:
                raise ValueError(f"Language code '{lang_code}' is not supported.")
    
    def _find_similar_languages(self, lang_code: str, max_suggestions: int = 3) -> List[str]:
        """Find languages that might be similar to the input."""
        from difflib import get_close_matches
        
        return get_close_matches(lang_code, self.available_languages, n=max_suggestions, cutoff=0.6)
    
    def download_glove(self, lang_code: str, file_type: str = "txt") -> str:
        """Download GloVe embeddings for a specific language."""
        # Validate language support
        self._validate_language(lang_code)
        
        repo_id = self.GLOVE_REPO_TEMPLATE.format(lang_code)
        filename = f"{lang_code}_embeddings.{file_type}"
        
        path = hf_hub_download(
            repo_id=repo_id,
            filename=filename,
            cache_dir=self.cache_dir
        )
        return path
    
    def download_graph(self, lang_code: str) -> str:
        """Download graph embeddings for a specific language."""
        # Validate language support
        self._validate_language(lang_code)

        if lang_code=="all":
            filename = f"ppmi_embeddings_{lang_code}.tar"
        
        filename = f"ppmi_embeddings_{lang_code}.txt"
        
        path = hf_hub_download(
            repo_id=self.GRAPH_REPO,
            filename=filename,
            cache_dir=self.cache_dir
        )
        return path