import numpy as np
from typing import List, Dict, Tuple
from sklearn.preprocessing import MinMaxScaler
from sklearn.linear_model import LinearRegression

class EmbeddingsOperator:
    """Handle operations on embeddings like loading, merging, and alignment."""
    
    def __init__(self):
        pass
    
    def normalize_embeddings(self, embeddings: np.ndarray, min_val: float, max_val: float) -> np.ndarray:
        """Normalize embeddings to a specified range.
        
        Args:
            embeddings: numpy array of embeddings to normalize
            min_val: minimum value for normalization
            max_val: maximum value for normalization
            
        Returns:
            Normalized embeddings array
        """
        scaler = MinMaxScaler(feature_range=(min_val, max_val))
        return scaler.fit_transform(embeddings)
    
    def read_embeddings_from_text(self, file_path: str, embedding_size: int = 300) -> Dict[str, np.ndarray]:
        """Read embeddings from a text file.
        
        Args:
            file_path: Path to embeddings file
            embedding_size: Size of embedding vectors
            
        Returns:
            Dictionary mapping words to their embeddings
        """
        embeddings = {}
        with open(file_path, 'r', encoding='utf-8') as file:
            for line in file:
                parts = line.strip().split(' ')
                embedding_start_index = len(parts) - embedding_size
                phrase = ' '.join(parts[:embedding_start_index])
                embedding = np.array([float(val) for val in parts[embedding_start_index:]])
                embeddings[phrase] = embedding
        return embeddings
    
    def svd_alignment(self, embeddings_dict_1: Dict[str, np.ndarray], 
                     embeddings_dict_2: Dict[str, np.ndarray]) -> Dict[str, np.ndarray]:
        """Align embeddings using SVD.
        
        Args:
            embeddings_dict_1: First embedding space
            embeddings_dict_2: Second embedding space
            
        Returns:
            Transformed first embedding space aligned with second space
        """
        common_words = [word for word in embeddings_dict_1 
                       if word in embeddings_dict_2 and len(embeddings_dict_1[word]) == 300]
        print('COMMON VOCABULARY between the embedding spaces:', len(common_words))
        
        # Prepare data for alignment
        embeddings_1 = np.array([embeddings_dict_1[word] for word in common_words])
        minimum = np.min(embeddings_1)
        maximum = np.max(embeddings_1)
        
        # Normalize embeddings
        embeddings_2 = np.array([embeddings_dict_2[word] for word in common_words])
        embeddings_2 = self.normalize_embeddings(embeddings_2, minimum, maximum)
        
        # Concatenate embeddings for SVD
        concatenated_embeddings = np.concatenate((embeddings_1, embeddings_2), axis=1)
        
        # Perform SVD on concatenated embeddings
        U, S, Vt = np.linalg.svd(concatenated_embeddings, full_matrices=False)
        reduced_embeddings = U[:, :300]
        
        # Fit linear regression to find transformation matrix
        model = LinearRegression()
        model.fit(embeddings_1, reduced_embeddings)
        transformation = model.coef_.T
        
        # Transform the first embedding space
        transformed_embeddings = {word: np.dot(embeddings_dict_1[word], transformation) 
                                for word in embeddings_dict_1}
        return transformed_embeddings
    
    def save_embeddings_to_text(self, embeddings: Dict[str, np.ndarray], file_path: str):
        """Save embeddings to a text file.
        
        Args:
            embeddings: Dictionary mapping words to their embeddings
            file_path: Path where to save the embeddings
        """
        with open(file_path, 'w', encoding='utf-8') as file:
            for word, vector in embeddings.items():
                vector_str = ' '.join(map(str, vector))
                file.write(f"{word} {vector_str}\n")
    
    def load_embeddings(self, filepath: str) -> Tuple[Dict[str, np.ndarray], int]:
        """Load embeddings from file."""
        embeddings = {}
        
        with open(filepath, 'r', encoding='utf-8') as f:
            first_line = f.readline().strip().split()
            if len(first_line) == 2:  # Word2Vec format
                vocab_size, dim = map(int, first_line)
                first_line = None
            else:
                dim = len(first_line) - 1
            
            def process_line(line):
                parts = line.strip().split()
                word = parts[0]
                vector = np.array([float(x) for x in parts[1:]], dtype=np.float32)
                return word, vector
            
            if first_line:
                word, vector = process_line(' '.join(first_line))
                embeddings[word] = vector
                
            for line in f:
                word, vector = process_line(line)
                embeddings[word] = vector
                
        return embeddings, dim